{-# LANGUAGE RecordWildCards #-}

import Affection
import qualified SDL
import qualified SDL.Raw as Raw
import qualified GEGL as G
import qualified BABL as B
import qualified Data.Map.Strict as M

import Foreign.Storable (peek)
import Foreign.C.Types  (CInt(..))

import Debug.Trace

-- main :: IO ()
-- main = withAllAffection $
--   withDefaultWindow "test" $ do
--     changeColor $ RGBA 255 255 255 255
--     clear
--     present
--     liftIO $ delaySec 2

main :: IO ()
main = do
  conf <- return $ AffectionConfig
    { initComponents = All
    , windowTitle    = "Affection: example00"
    , windowConfig   = SDL.defaultWindow
    , drawLoop       = draw
    , updateLoop     = update
    , loadState      = load
    }
  withAffection conf

type UserData = M.Map String G.GeglNode

load :: SDL.Surface -> IO UserData
load _ = do
  traceM "loading"
  root <- G.gegl_node_new
  traceM "new root node"
  checkerboard <- G.gegl_node_new_child root $ G.checkerboardOperation
    [ G.Property "color1" $ G.PropertyColor $ G.RGBA 0.4 0.4 0.4 1
    , G.Property "color2" $ G.PropertyColor $ G.RGBA 0.6 0.6 0.6 1
    ]
  traceM "checkerboard"
  myMap <- return $ M.fromList
    [ ("root"        , root)
    , ("checkerboard", checkerboard)
    ]
  traceM "loading complete"
  return myMap

draw :: AffectionState (AffectionData UserData) IO ()
draw = do
  traceM "drawing"
  AffectionData{..} <- get
  liftIO $ SDL.lockSurface drawSurface
  pixels <- liftIO $ SDL.surfacePixels drawSurface
  let SDL.Surface rawSurfacePtr _ = drawSurface
  rawSurface <- liftIO $ peek rawSurfacePtr
  pixelFormat <- liftIO $ peek $ Raw.surfaceFormat rawSurface
  format <- liftIO $ (B.babl_format $ B.PixelFormat B.R'G'B'A B.CFu8)
  SDL.V2 (CInt rw) (CInt rh) <- SDL.surfaceDimensions drawSurface
  let (w, h) = (fromIntegral rw, fromIntegral rh)
  liftIO $ G.gegl_node_blit
    (userState M.! "checkerboard" :: G.GeglNode)
    1
    (G.GeglRectangle 0 0 w h)
    format
    pixels
    (fromIntegral (Raw.pixelFormatBytesPerPixel pixelFormat) * w)
    [G.GeglBlitDefault]
  liftIO $ SDL.unlockSurface drawSurface
  liftIO $ SDL.updateWindowSurface drawWindow

update :: Double -> AffectionState (AffectionData UserData) IO ()
update sec = do
  traceM "updating"
  -- traceM $ show sec
  liftIO $ delaySec 5
  ud@AffectionData{..} <- get
  put $ ud
    { quitEvent = True
    }
